#include "IndividualPieces.h"

/* Piece colours in 0xRRGGBBAA format */
Uint32 Piece_I::colour = 0x00FFFEFF;
Uint32 Piece_J::colour = 0x1F00FAFF;
Uint32 Piece_L::colour = 0xFFA42CFF;
Uint32 Piece_O::colour = 0xFEFF3AFF;
Uint32 Piece_S::colour = 0x6DFF35FF;
Uint32 Piece_T::colour = 0x83007DFF;
Uint32 Piece_Z::colour = 0xFF0020FF;

/* Matrices used in Piece::rotate function */
const char Piece_J::rotation_matrix[24] = {
    0, -1, 0, 1, -1, 1,
    0, 1, 0, -1, 1, -1,
    -1, 0, 1, 0, 1, 1,
    1, 0, -1, 0, -1, -1
};
const char Piece_L::rotation_matrix[24] = {
    0, -1, 0, 1, 1, 1,
    0, 1, 0, -1, -1, -1,
    -1, 0, 1, 0, 1, -1,
    1, 0, -1, 0, -1, 1
};
const char Piece_S::rotation_matrix[24] = {
    0, -1, 1, 0, 1, 1,
    0, 1, -1, -1, -1, 0,
    -1, 0, 0, -1, 1, -1,
    1, 0, 0, 1, -1, 1
};
const char Piece_T::rotation_matrix[24] = {
    0, -1, -1, 0, 1, 0,
    0, 1, 1, 0, -1, 0,
    -1, 0, 0, -1, 0, 1,
    1, 0, 0, 1, 0, -1
};

const char Piece_Z::rotation_matrix[24] = {
    0, -1, -1, 0, -1, 1, 
    0, 1, 1, -1, 1, 0,
    -1, 0, 0, 1, 1, 1, 
    1, 0, 0, -1, -1, -1
};

/**
 * @brief Piece_I ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_I::Piece_I(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window) {
    blocks.push_back(new Block(x_pos-2, 0, colour, window));
    blocks.push_back(new Block(x_pos-1, 0, colour, window));
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos+1, 0, colour, window));
}

/**
 * @brief Piece_J ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_J::Piece_J(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window, Piece_J::rotation_matrix) {
    blocks.push_back(new Block(x_pos-1, 0, colour, window));
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos+1, 0, colour, window));
    blocks.push_back(new Block(x_pos+1, 1, colour, window));
}

/**
 * @brief Piece_L ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_L::Piece_L(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window, Piece_L::rotation_matrix) {
    blocks.push_back(new Block(x_pos+1, 0, colour, window));
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos-1, 0, colour, window));
    blocks.push_back(new Block(x_pos-1, 1, colour, window));
}

/**
 * @brief Piece_O ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_O::Piece_O(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window) {
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos+1, 0, colour, window));
    blocks.push_back(new Block(x_pos, 1, colour, window));
    blocks.push_back(new Block(x_pos+1, 1, colour, window));
}

/**
 * @brief Piece_S ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_S::Piece_S(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window, Piece_S::rotation_matrix) {
    blocks.push_back(new Block(x_pos-1, 1, colour, window));
    blocks.push_back(new Block(x_pos, 1, colour, window));
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos+1, 0, colour, window));
}

/**
 * @brief Piece_T ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_T::Piece_T(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window, Piece_T::rotation_matrix) {
    blocks.push_back(new Block(x_pos, 1, colour, window));
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos-1, 0, colour, window));
    blocks.push_back(new Block(x_pos+1, 0, colour, window));
}

/**
 * @brief Piece_Z ctor
 * @param x_pos - x position within TetrisWindow
 * @param containing_window - containing TetrisWindow
 */
Piece_Z::Piece_Z(int x_pos, const TetrisWindow* containing_window) : Piece(containing_window, Piece_Z::rotation_matrix) {
    blocks.push_back(new Block(x_pos+1, 1, colour, window));
    blocks.push_back(new Block(x_pos, 1, colour, window));
    blocks.push_back(new Block(x_pos, 0, colour, window));
    blocks.push_back(new Block(x_pos-1, 0, colour, window));
}

/**
 * @brief Overridden rotate function
 * @details Piece_I doesn't rotate nicely around 1 block with the Piece::rotate() function
 */
void Piece_I::rotate() {
    int x = blocks[1]->get_x();
    int y = blocks[1]->get_y();
    
    if(blocks[0]->get_x() == blocks[1]->get_x()) { // vertical
        if(is_clear_to_rotate(x, blocks[1], x-1, y, x+1, y, x+2, y)) {
            blocks[0]->set(x-1, y);
            blocks[2]->set(x+1, y);
            blocks[3]->set(x+2, y);
        }
    }
    else { // horizontal
        if(is_clear_to_rotate(x, blocks[1], x, y-1, x, y+1, x, y+2)) {
            blocks[0]->set(x, y-1);
            blocks[2]->set(x, y+1);
            blocks[3]->set(x, y+2);
        }
    }
}

/**
 * @brief Overridden rotate function
 */
void Piece_O::rotate() {
    /* nice, simple algorithm */
}
