#ifndef TETRISWINDOW_H_INCLUDED
#define TETRISWINDOW_H_INCLUDED

#include <algorithm>
#include <ctime>
#include <vector>

#include <SDL2/SDL.h>
#include "SDL2_gfxPrimitives.h"

#include "Block.h"
#include "MusicPlayer.h"
#include "Piece.h"
#include "SoundPlayer.h"
#include "text.h"

#define SOUND_ON

class Block;
class Piece;

extern SDL_Renderer* sdlRenderer;
extern SDL_Event ev;

/**
 * @class TetrisWindow
 * @author zsmb
 * @brief A window that contains a game of tetris
 */
class TetrisWindow {
    const unsigned x;                        //!< x coordinate from the left, in pixels
    const unsigned y;                        //!< y coordinate from the top, in pixels
    const unsigned vert_blocks, hori_blocks; //!< Number of blocks in the window in each direction
    const unsigned block_size;               //!< Size of a single block in the window
    
    std::vector<Block*> blocks;              //!< Blocks that the window is holding
    std::vector<std::vector<Block*>> rows;   //!< Subsets of previous collection, by row index
    
    Piece* current_piece; //!< The current moving piece of the tetris game
    
    unsigned score;       //!< The score achieved by the player in current game
    
    bool sound_enabled;   //!< Tells you if sound is enabled..
    
private:
    Piece* random_piece() const;
    bool replace_current_piece();
    
    bool completed(std::vector<Block*> row);
    
public:
    TetrisWindow(unsigned x_pos, unsigned y_pos, unsigned vert_blocks, unsigned horiz_blocks, unsigned block_size, bool music);
    ~TetrisWindow();
    
    unsigned get_block_size() const;
    unsigned get_x1(unsigned x) const;
    unsigned get_y1(unsigned y) const;
    
    bool is_available(unsigned x, unsigned y) const;
    
    void draw() const;
    unsigned loop();
};

#endif // TETRISWINDOW_H_INCLUDED
