/**
 * @mainpage Tetris - a project by zsmb
 * @section intro Introduction
 * It's Tetris. There's music. There's even a sound effect when you clear rows. It's pretty cool.
 * @section projectpage Project page
 * You can find the page for this project at http://zsmb.co/projects/tetris/
 */

#include <iostream>

#include <SDL2/SDL.h>
#include "SDL2_gfxPrimitives.h"

#include "Block.h"
#include "HighScores.h"
#include "IndividualPieces.h"
#include "Menu.h"
#include "Piece.h"
#include "TetrisWindow.h"
#include "TextDisplay.h"

SDL_Event ev;
SDL_Renderer *sdlRenderer;

const int WIDTH = 540;
const int HEIGHT = 720;

void draw_bg() {
    boxColor(sdlRenderer, 0, 0, WIDTH, HEIGHT, 0x293134FF);
}

int main(int argc, char *argv[]) {
    SDL_Window *sdlWindow;
    try {
        sdlWindow = SDL_CreateWindow("Tetris",
                                     SDL_WINDOWPOS_UNDEFINED,
                                     SDL_WINDOWPOS_UNDEFINED,
                                     WIDTH, HEIGHT,
                                     0);
        if(sdlWindow == NULL) throw 1;
        
        sdlRenderer = SDL_CreateRenderer(sdlWindow, -1, 0);
        if(sdlRenderer == NULL) throw 1;
    }
    catch(int error) {
        SDL_Quit();
        return error;
    }
    
    Menu main_menu(100, 120, 340);
    main_menu.add_button(SDLK_p, "Play", "P");          // 0
    main_menu.add_button(SDLK_s, "High Scores", "S");   // 1
    main_menu.add_button(SDLK_t, "Toggle Music", "T");  // 2
    main_menu.add_button(SDLK_h, "Help", "H");          // 3
    main_menu.add_button(SDLK_a, "About", "A");         // 4
    main_menu.add_button(SDLK_e, "Exit", "E");          // 5
    
    HighScores hs(100, 70, 340);
    
    TextDisplay help(70, "help.txt");
    TextDisplay about(70, "about.txt");
    
    MusicPlayer mp("OctoNote - Korobeiniki Tetris Remix.ogg");
    
    bool music = false;
    bool quit = false;
    
    try {
        draw_bg();
        main_menu.draw();
        SDL_RenderPresent(sdlRenderer);        
        
        while(SDL_WaitEvent(&ev) && !quit) {
            int command = -1;
            
            switch(ev.type) {
                case SDL_QUIT:
                    quit = true;
                    break;
                case SDL_MOUSEBUTTONDOWN:
                    if(ev.button.button == SDL_BUTTON_LEFT) {
                        command = main_menu.click(ev.button.x, ev.button.y);
                    }
                    break;
                case SDL_KEYDOWN:
                    command = main_menu.press(ev.key.keysym.sym);
                    break;
            }
            
            switch(command) {
                case 5: /* Exit */
                    quit = true;
                    break;
                case 4: /* About */
                    draw_bg();
                    about.loop();
                    break;
                case 3: /* Help */
                    draw_bg();
                    help.loop();
                    break;
                case 2: /* Toggle Music */
                    if(music) {
                        mp.stop();
                        music = false;
                    }
                    else {
                        mp.play();
                        music = true;
                    }
                    break;
                case 1: /* High Scores */
                    draw_bg();
                    hs.loop();
                    break;
                case 0: /* Play */
                    mp.stop();
                    draw_bg();
                    
                    TetrisWindow tetris_window(30, 30, 16, 22, 30, music);
                    hs.add(tetris_window.loop());
                    
                    if(music) {
                        mp.play();
                    }
                    break;
            }
            
            draw_bg();
            main_menu.draw();
            SDL_RenderPresent(sdlRenderer);
        }
    }
    catch(std::exception& x) {
        std::cerr << "Error: " << x.what() << std::endl;
    }
    
    SDL_Quit();
    return 0;
}
